@@ -2,9 +2,9 @@
  *
  * Copyright (C) 2015 Red Hat, Inc.
  *
- * This program is free software: you can redistribute it and/or modify
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Author: Gris Ge <fge@redhat.com>
  *
@@ -437,6 +438,13 @@ _fill_supported_system_id_hash (lsm_connect *lsm_conn)
   return rc;
 }
 
+static void
+_free_lsm_volume_record (gpointer data)
+{
+  lsm_volume_record_free ((lsm_volume *) data);
+  return;
+}
+
 /*
  * Return an array of lsm_volume which system_id is in supported_sys_id_hash.
  */
@@ -461,7 +469,7 @@ _get_supported_lsm_volumes (lsm_connect *lsm_conn)
     }
 
   lsm_vol_array =
-    g_ptr_array_new_full (0, (GDestroyNotify) lsm_volume_record_free);
+    g_ptr_array_new_full (0, (GDestroyNotify) _free_lsm_volume_record);
 
   for (i = 0; i < lsm_vol_count; ++i)
     {
@@ -500,6 +508,13 @@ _get_supported_lsm_volumes (lsm_connect *lsm_conn)
   return lsm_vol_array;
 }
 
+static void
+_free_lsm_pool_record (gpointer data)
+{
+  lsm_pool_record_free ((lsm_pool *) data);
+  return;
+}
+
 /*
  * Return an array of lsm_pool which system_id is in supported_sys_id_hash.
  */
@@ -524,7 +539,7 @@ _get_supported_lsm_pls (lsm_connect *lsm_conn)
     }
 
   lsm_pl_array =
-    g_ptr_array_new_full (0, (GDestroyNotify) lsm_pool_record_free);
+    g_ptr_array_new_full (0, (GDestroyNotify) _free_lsm_pool_record);
 
   for (i = 0; i < lsm_pl_count; ++i)
     {
