@@ -34,7 +34,8 @@ GType                     udisks_daemon_get_type             (void) G_GNUC_CONST
 UDisksDaemon             *udisks_daemon_new                  (GDBusConnection *connection,
                                                               gboolean         disable_modules,
                                                               gboolean         force_load_modules,
-                                                              gboolean         uninstalled);
+                                                              gboolean         uninstalled,
+                                                              gboolean         enable_tcrypt);
 GDBusConnection          *udisks_daemon_get_connection        (UDisksDaemon    *daemon);
 GDBusObjectManagerServer *udisks_daemon_get_object_manager    (UDisksDaemon    *daemon);
 UDisksMountMonitor       *udisks_daemon_get_mount_monitor     (UDisksDaemon    *daemon);
@@ -51,6 +52,7 @@ UDisksConfigManager      *udisks_daemon_get_config_manager    (UDisksDaemon    *
 gboolean                  udisks_daemon_get_disable_modules   (UDisksDaemon    *daemon);
 gboolean                  udisks_daemon_get_force_load_modules(UDisksDaemon    *daemon);
 gboolean                  udisks_daemon_get_uninstalled       (UDisksDaemon    *daemon);
+gboolean                  udisks_daemon_get_enable_tcrypt     (UDisksDaemon    *daemon);
 
 /**
  * UDisksDaemonWaitFunc:
@@ -171,6 +173,9 @@ gboolean                 udisks_daemon_launch_threaded_job_sync (UDisksDaemon
                                                                  GCancellable          *cancellable,
                                                                  GError               **error);
 
+void                     udisks_bd_thread_set_progress_for_job  (UDisksJob             *job);
+void                     udisks_bd_thread_disable_progress      (void);
+
 /* Return value and *uuid_ret must be freed with g_free.  If return
    value is NULL, *uuid has not been changed.
  */
