@@ -1902,6 +1902,12 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta  *drive,
   if (object == NULL)
     goto out;
   _drive = udisks_object_peek_drive (UDISKS_OBJECT (object));
+  if (_drive == NULL)
+    {
+      g_set_error (&local_error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Failed to get Drive interface for object");
+      goto out;
+    }
 
   block_object = udisks_linux_drive_object_get_block (object, FALSE);
   if (block_object == NULL)
@@ -2152,7 +2158,7 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta  *drive,
     {
       udisks_notice ("Error securely erasing %s (%s): %s (%s, %d)",
                      device_file,
-                     udisks_drive_get_id (_drive),
+                     _drive ? udisks_drive_get_id (_drive) : "",
                      local_error->message, g_quark_to_string (local_error->domain), local_error->code);
     }
 
@@ -2201,7 +2207,6 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
   uid_t caller_uid;
   gid_t caller_gid;
   gboolean enhanced = FALSE;
-  UDisksInhibitCookie *inhibit_cookie = NULL;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -2256,8 +2261,6 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
                                                     invocation))
     goto out;
 
-  inhibit_cookie = udisks_daemon_util_inhibit_system_sync (N_("Formatting Device"));
-
   if (!udisks_linux_drive_ata_secure_erase_sync (drive, caller_uid, enhanced, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
@@ -2267,10 +2270,7 @@ handle_security_erase_unit (UDisksDriveAta        *_drive,
 
   udisks_linux_block_object_reread_partition_table (UDISKS_LINUX_BLOCK_OBJECT (block_object));
 
-  udisks_drive_ata_complete_security_erase_unit (_drive, invocation);
-
  out:
-  udisks_daemon_util_uninhibit_system_sync (inhibit_cookie);
   g_clear_object (&block_object);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
