@@ -178,8 +178,7 @@ udisks_linux_filesystem_update (UDisksLinuxFilesystem  *filesystem,
   udisks_filesystem_set_mount_points (UDISKS_FILESYSTEM (filesystem),
                                       (const gchar *const *) p->pdata);
   g_ptr_array_free (p, TRUE);
-  g_list_foreach (mounts, (GFunc) g_object_unref, NULL);
-  g_list_free (mounts);
+  g_list_free_full (mounts, g_object_unref);
 
   dev = udisks_linux_block_object_get_device_file (object);
   type = g_udev_device_get_property (device->udev_device, "ID_FS_TYPE");
@@ -2249,7 +2248,7 @@ handle_resize (UDisksFilesystem      *filesystem,
       goto out;
     }
 
-  /* TODO: implement progress parsing for udisks_job_set_progress(_valid) */
+  udisks_bd_thread_set_progress_for_job (UDISKS_JOB (job));
   if (! bd_fs_resize (udisks_block_get_device (block), size, &error))
     {
       g_dbus_method_invocation_return_error (invocation,
@@ -2266,6 +2265,7 @@ handle_resize (UDisksFilesystem      *filesystem,
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  udisks_bd_thread_disable_progress ();
   g_clear_object (&object);
   g_free (required_utility);
   g_clear_error (&error);
@@ -2407,7 +2407,7 @@ handle_repair (UDisksFilesystem      *filesystem,
       goto out;
     }
 
-  /* TODO: implement progress parsing for udisks_job_set_progress(_valid) */
+  udisks_bd_thread_set_progress_for_job (UDISKS_JOB (job));
   ret = bd_fs_repair (udisks_block_get_device (block), &error);
   if (error)
     {
@@ -2425,6 +2425,7 @@ handle_repair (UDisksFilesystem      *filesystem,
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  udisks_bd_thread_disable_progress ();
   g_clear_object (&object);
   g_free (required_utility);
   g_clear_error (&error);
@@ -2566,7 +2567,7 @@ handle_check (UDisksFilesystem      *filesystem,
       goto out;
     }
 
-  /* TODO: implement progress parsing for udisks_job_set_progress(_valid) */
+  udisks_bd_thread_set_progress_for_job (UDISKS_JOB (job));
   ret = bd_fs_check (udisks_block_get_device (block), &error);
   if (error)
     {
@@ -2584,6 +2585,7 @@ handle_check (UDisksFilesystem      *filesystem,
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  udisks_bd_thread_disable_progress ();
   g_clear_object (&object);
   g_free (required_utility);
   g_clear_error (&error);
