@@ -28,6 +28,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 #include <mntent.h>
 
 #include <glib.h>
@@ -37,6 +38,7 @@
 #include "udisksmountmonitor.h"
 #include "udisksmount.h"
 #include "udisksprivate.h"
+#include "udisksdaemonutil.h"
 
 /* build a %Ns format string macro with N == PATH_MAX */
 #define xstr(s) str(s)
@@ -118,8 +120,7 @@ udisks_mount_monitor_finalize (GObject *object)
   if (monitor->swaps_watch_source != NULL)
     g_source_destroy (monitor->swaps_watch_source);
 
-  g_list_foreach (monitor->mounts, (GFunc) g_object_unref, NULL);
-  g_list_free (monitor->mounts);
+  g_list_free_full (monitor->mounts, g_object_unref);
 
   if (G_OBJECT_CLASS (udisks_mount_monitor_parent_class)->finalize != NULL)
     G_OBJECT_CLASS (udisks_mount_monitor_parent_class)->finalize (object);
@@ -238,8 +239,7 @@ reload_mounts (UDisksMountMonitor *monitor)
 
   udisks_mount_monitor_ensure (monitor);
 
-  old_mounts = g_list_copy (monitor->mounts);
-  g_list_foreach (old_mounts, (GFunc) g_object_ref, NULL);
+  old_mounts = g_list_copy_deep (monitor->mounts, (GCopyFunc) udisks_g_object_ref_copy, NULL);
 
   udisks_mount_monitor_invalidate (monitor);
   udisks_mount_monitor_ensure (monitor);
@@ -262,8 +262,7 @@ reload_mounts (UDisksMountMonitor *monitor)
       g_signal_emit (monitor, signals[MOUNT_ADDED_SIGNAL], 0, mount);
     }
 
-  g_list_foreach (old_mounts, (GFunc) g_object_unref, NULL);
-  g_list_free (old_mounts);
+  g_list_free_full (old_mounts, g_object_unref);
   g_list_free (cur_mounts);
   g_list_free (removed);
   g_list_free (added);
@@ -306,14 +305,25 @@ udisks_mount_monitor_constructed (GObject *object)
   if (monitor->mounts_channel != NULL)
     {
       monitor->mounts_watch_source = g_io_create_watch (monitor->mounts_channel, G_IO_ERR);
+#if __GNUC__ >= 8
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
+/* parameters of the callback depend on the source and can be different
+ * from the required "generic" GSourceFunc, see:
+ * https://developer.gnome.org/glib/stable/glib-The-Main-Event-Loop.html#g-source-set-callback
+ */
       g_source_set_callback (monitor->mounts_watch_source, (GSourceFunc) mounts_changed_event, monitor, NULL);
+#if __GNUC__ >= 8
+#pragma GCC diagnostic pop
+#endif
       g_source_attach (monitor->mounts_watch_source, g_main_context_get_thread_default ());
       g_source_unref (monitor->mounts_watch_source);
     }
   else
     {
       g_error ("No /proc/self/mountinfo file: %s", error->message);
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
   error = NULL;
@@ -321,7 +331,18 @@ udisks_mount_monitor_constructed (GObject *object)
   if (monitor->swaps_channel != NULL)
     {
       monitor->swaps_watch_source = g_io_create_watch (monitor->swaps_channel, G_IO_ERR);
+#if __GNUC__ >= 8
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
+/* parameters of the callback depend on the source and can be different
+ * from the required "generic" GSourceFunc, see:
+ * https://developer.gnome.org/glib/stable/glib-The-Main-Event-Loop.html#g-source-set-callback
+ */
       g_source_set_callback (monitor->swaps_watch_source, (GSourceFunc) swaps_changed_event, monitor, NULL);
+#if __GNUC__ >= 8
+#pragma GCC diagnostic pop
+#endif
       g_source_attach (monitor->swaps_watch_source, g_main_context_get_thread_default ());
       g_source_unref (monitor->swaps_watch_source);
     }
@@ -332,7 +353,7 @@ udisks_mount_monitor_constructed (GObject *object)
           udisks_warning ("Error opening /proc/swaps file: %s (%s, %d)",
                           error->message, g_quark_to_string (error->domain), error->code);
         }
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
   if (G_OBJECT_CLASS (udisks_mount_monitor_parent_class)->constructed != NULL)
@@ -361,8 +382,7 @@ udisks_mount_monitor_invalidate (UDisksMountMonitor *monitor)
 {
   monitor->have_data = FALSE;
 
-  g_list_foreach (monitor->mounts, (GFunc) g_object_unref, NULL);
-  g_list_free (monitor->mounts);
+  g_list_free_full (monitor->mounts, g_object_unref);
   monitor->mounts = NULL;
 }
 
@@ -613,7 +633,7 @@ udisks_mount_monitor_ensure (UDisksMountMonitor *monitor)
     {
       udisks_warning ("Error getting mounts: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
   error = NULL;
@@ -621,7 +641,7 @@ udisks_mount_monitor_ensure (UDisksMountMonitor *monitor)
     {
       udisks_warning ("Error getting swaps: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
   monitor->have_data = TRUE;
